---
title: The World's Ugliest Mastodon Client
date: 07-10-2019
...

Or How To Send a Mastodon Post Using `curl`

The idea behind this is that one could easily make a [Mastodon](https://joinmastodon.org/) post from a Linux terminal without having to really interact with a UI. Yes, I mainly wrote this because I was bored, but I do feel that a pure "command-line" mastodon client: where you don't have to memorise a million keyboard shortcuts to ninja mode-switch through an ncurses-style text-interface is quite elegant[^1]. It is programs that can interact through arguments and `stdin`/`stdout` that make scripting languages really powerful. 

Of course, with that comes the ugliness of small programs that require many, many dependencies. The universality of bash on Linux systems seems attractive, until you realise how many people use zsh or fish or whatever terminal is cool right now, how its non-standard on BSD systems and how the need to call out to other software means you have to be aware of various dependencies' availability and versions. Yet this script does so little it could really just be a couple of lines in a `.bashrc` profile. I am making a note of this because availability is the only real reason to use bash given how lacking in abstraction and readability it is. Although perhaps I just haven't learnt the idioms yet. 

More importantly, I've never really worked with JSON APIs before and its surprisingly easy and jq seems like a really cool tool for working with JSON. 

```bash
#!/bin/bash

# Simple bash script that will send all its arguments as a mastodon
# post.
# On first run it will ask for a username and password and
# generate an API key which it will store at the end of this file.

# Dependencies:
#
# bash
# jq    : JSON Parser
# curl  : API requests and POSTs

## config
mastoinstance="mastodon.social"
defaultstatusvis="public" # change to "direct" or "unlisted"
statuslang="eng"
storepassword="True"
## end of config

THISFILE=$(pwd)/toot.sh

# Check whether the api key has already been generated
if [ $(tail -1 $THISFILE | grep "ACCESSTOKEN") ]
then
    accesstoken=$(tail -1 $THISFILE | cut -d':' -f2)
else
    # Otherwise generate a new key with username and password
    printf "Please enter your email: "
    read email
    printf "Please enter your password: "
    read pass

    # Register the app as per
    #   https://docs.joinmastodon.org/api/rest/apps/#post-api-v1-apps

    meta=$(curl -X POST -sS "https://$mastoinstance/api/v1/apps"\
        -d  "client_name=tinyscript&redirect_uris=urn:ietf:wg:"\
        "oauth:2.0:oob&scopes=write follow read")

    # Use jq to parse the json response
    key=$(echo "$meta" | jq '.vapid_key'| cut -d'"' -f 2)
    client_id=$(echo "$meta" | jq '.client_id'| cut -d'"' -f 2)
    client_secret=$(echo "$meta" | jq '.client_secret'| cut -d'"' -f 2)
    key=$(echo "$meta" | jq '.vapid_key'| cut -d'"' -f 2)

    # Login with Oauth2 to get an api key
    login=$(curl -X POST -d "scope=read follow write&client_id=$client_id"\
        "&client_secret=$client_secret&grant_type=password&username="\
        "$email&password=$pass" -Ss https://mastodon.social/oauth/token)

    # Use jq again to extract the access token
    accesstoken=$(echo "$login" | jq '.access_token'| cut -d'"' -f 2)

    if [ storepassword == "True" ]
    then
        # Store the access token at the end of this file
        # Security: 100
        printf "#ACCESSTOKEN:$accesstoken" >> $THISFILE
    fi

    echo "Logged in (probably) successfully :)"
fi

# Check there is actually a message
if [ "$1" != "" ]
then
    # Actually send the message
    statustext="$@"
    header="Authorization: Bearer $accesstoken"
    url="https://$mastoinstance/api/v1/statuses"
    response=$(curl --header "$header" -sS $url -X POST \
        -d "status=$statustext&language=$statuslang&visibility="\
        "$defaultstatusvis" | jq '.content')
    echo "Sent toot: $response"
else
    echo "Nothing to write :("
fi

    ### Below Here your access token is stored in one line ###
```

[^1]: To be clear, this is a statement about my general overwhelment at command line GUIs. I haven't used any mastodon terminal program. 
