---
title: My Very Basic Site Generator
date: 13-12-2019
...

I took a few hours today to rework the experimental Makefile and Python script into just a python script which works somewhat solidly, albeit still for a small set of features compared to full-blown static site generators like Hugo. 

I'm quite satisfied with it, it does the core job I need it to do and does it well enough.

The site generator's behaviour is very simple, it recurses through each directory in `src/` and builds what it sees. Anything that it does not understand is copied to the output directory, unless the config file explicitly tells it not to.

This time the source is online: [git.topost.net](https://git.topost.net/alistair/blog/src/branch/master)

## Features

### Configuration File

TOML and YAML are silly and overcomplicated so config is done through a JSON file (see `src/generator/config.json`). 

`ignorexts`, `ignorefiles`, and `ignoredirs` provide file extensions, file names, and _specific_ directories  respectively, to not copy to the destination directory.

The same labels in the `dir_index` context specify things to be ignored by the directory-index-page generator. 

### Templates

It has very basic support for templating, any file can have a `.template` appended to its filename, and the script will read it and try to substitute any variables in the file. Current supported substitutions are:

- Any variable name in root of the config file, for example `${baseurl}` evaluates to `https://alistairmichael.com`
- The `${include:relative/file/path}` function inserts the context of the provided file.
- The `${postlist:relative/file/path}` function generates a list of posts as seen on the front page. This can be done for any directory on the site. 

The file generated from `filename.ext.template` is then written to `filename.ext` so the program can continue processing it, for example generating html from it. Anything ending in `.template` is not copied to the public directory. 

### Markdown

It utilises Pandoc, which has a very extensive dialect with useful things like code highlighting, footnotes, tables-of-contents. These days I use pandoc for everything that doesn't need Latex, it really is fantastic. 

### Pandoc HTML Templates

The `pandoc` command is always provided with a pandoc template, the script chooses the one closest to the file it is currently processing by walking up the file tree. These are named `template.html`. For bonus templating make a `template.html.template` which can use the python scripts substitution, and pandoc's substitution. This is how I structure and style the website.

